#include "stdafx.h"
#include "LetterSelectionDialog.h"
#include "SettingsDialog.h"
#include <bzswin/i18n.h>

LRESULT CLetterSelectionDialog::OnInitDialog( UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled )
{
	m_cbLetters.m_hWnd = GetDlgItem(IDC_DRIVELETTER);

	if (m_bForciblyInvoked)
		::EnableWindow(GetDlgItem(IDC_AUTOLETTERS), FALSE);

	LocalizeLabels();

	DisplayParams();
	return 0;
}

LRESULT CLetterSelectionDialog::OnClose( UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled )
{
	EndDialog(IDCANCEL);
	return 0;
}

LRESULT CLetterSelectionDialog::OnBnClickedCancel( WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/ )
{
	EndDialog(IDCANCEL);
	return 0;
}

LRESULT CLetterSelectionDialog::OnBnClickedOk( WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/ )
{
	TCHAR tsz[3] = {0,};
	if (m_cbLetters.GetLBTextLen(m_cbLetters.GetCurSel()) >= _countof(tsz))
	{
		EndDialog(IDCANCEL);
		return 0;
	}
	m_cbLetters.GetLBText(m_cbLetters.GetCurSel(), tsz);
	m_DriveLetter = (char)(tsz[0]);

	if (SendDlgItemMessage(IDC_AUTOLETTERS, BM_GETCHECK) == BST_CHECKED)
		m_RegParams.DriveLetterPolicy = drvlStartingFromGiven;

	m_RegParams.StartingDriveLetterIndex = m_DriveLetter - 'A';
	m_RegParams.DisableAutorun = (SendDlgItemMessage(IDC_NOAUTORUN, BM_GETCHECK) == BST_CHECKED);

	if (!m_bForciblyInvoked)
		m_RegParams.SaveToRegistry();

	EndDialog(IDOK);
	return 0;
}

void CLetterSelectionDialog::DisplayParams()
{
	m_cbLetters.Clear();

	DWORD dwLetterMask = GetLogicalDrives();
	unsigned char ch = 'C';
	TCHAR tsz[] = _T("X:");
	bool selUpdated = false;
	for (unsigned i = (1 << (ch - 'A')); ch <= 'Z'; ch++, i <<= 1)
		if (!(dwLetterMask & i))
		{
			tsz[0] = ch, m_cbLetters.InsertString(-1, tsz);
			if (((unsigned char)(ch - 'A') >= m_RegParams.StartingDriveLetterIndex) && !selUpdated)
				m_cbLetters.SetCurSel(m_cbLetters.GetCount() - 1), selUpdated = true;
		}

	if (!m_cbLetters.GetCount())
	{
		MessageBox(_TR(IDS_NOFREELETTERS, "No free drive letters found!"), NULL, MB_ICONERROR);
		EndDialog(IDCANCEL);
	}

	if (!selUpdated)
		m_cbLetters.SetCurSel(0);
	m_cbLetters.SetFocus();

	SendDlgItemMessage(IDC_NOAUTORUN, BM_SETCHECK, m_RegParams.DisableAutorun ? BST_CHECKED : BST_UNCHECKED);
	SendDlgItemMessage(IDC_AUTOLETTERS, BM_SETCHECK, (m_RegParams.DriveLetterPolicy != drvlAskEveryTime) ? BST_CHECKED : BST_UNCHECKED);
}

LRESULT CLetterSelectionDialog::OnBnClickedSettings( WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/ )
{
	CSettingsDialog dlg;
	dlg.DoModal();
	LocalizeLabels();

	m_RegParams.LoadFromRegistry();
	DisplayParams();
	return 0;
}

void CLetterSelectionDialog::LocalizeLabels()
{
	LOCALIZE_DIALOG(IDS_MOUNTIMAGE, IDD_SELECTFOLDER, m_hWnd);
	LOCALIZE_DLGITEM(IDS_AUTOLETTERS, IDC_AUTOLETTERS, m_hWnd);
	LOCALIZE_DLGITEM(IDS_NOAUTORUN, IDC_NOAUTORUN, m_hWnd);
	LOCALIZE_DLGITEM(IDS_OK, IDOK, m_hWnd);
	LOCALIZE_DLGITEM(IDS_CANCEL, IDCANCEL, m_hWnd);
	LOCALIZE_DLGITEM(IDS_ELLIPSIS, IDC_SETTINGS, m_hWnd);
	LOCALIZE_DLGITEM(IDS_CHOOSELETTER, IDC_CHOOSELETTER, m_hWnd);
}