#include "stdafx.h"
#include "../VirtualCDCtl/VirtualCDClient.h"
#include <newdev.h>
#include "LetterSelectionDialog.h"
#include "SettingsDialog.h"
#include <bzswin/i18n.h>

using namespace std;
using namespace BazisLib;

CAppModule _Module;

int AppMain();

int __stdcall _tWinMain(HINSTANCE hInstance, HINSTANCE, LPTSTR lpCmdLine, int)
{
	HRESULT hRes = ::CoInitialize(NULL);
	// If you are running on NT 4.0 or higher you can use the following call instead to 
	// make the EXE free threaded. This means that calls come in on a random RPC thread.
	//	HRESULT hRes = ::CoInitializeEx(NULL, COINIT_MULTITHREADED);
	ATLASSERT(SUCCEEDED(hRes));

	// this resolves ATL window thunking problem when Microsoft Layer for Unicode (MSLU) is used
	::DefWindowProc(NULL, 0, 0, 0L);

	AtlInitCommonControls(ICC_BAR_CLASSES);	// add flags to support other controls

	hRes = _Module.Init(NULL, hInstance);
	ATLASSERT(SUCCEEDED(hRes));

#ifdef BAZISLIB_LOCALIZATION_ENABLED
	RegistryParams params;
	params.LoadFromRegistry();
	InitializeLNGBasedTranslationEngine(params.Language);
#endif

	int nRet = AppMain();

#ifdef BAZISLIB_LOCALIZATION_ENABLED
	ShutdownTranslationEngine();
#endif

	_Module.Term();
	::CoUninitialize();
	return nRet;
}

int AppMain()
{
	int argc;
	LPWSTR *ppArgv = CommandLineToArgvW(GetCommandLineW(), &argc);
	LPWSTR lpDiskPath = NULL;
	bool ShowSettingsDialog = false, ForceMountOptions = false;

	for (int i = 1; i < argc; i++)
	{
		if (ppArgv[i][0] != '/')
		{
			if (!lpDiskPath)
				lpDiskPath = ppArgv[i];
		}
		else
		{
			if (!_wcsicmp(ppArgv[i], L"/settings"))
				ShowSettingsDialog = true;
			else if (!_wcsicmp(ppArgv[i], L"/ltrselect"))
				ForceMountOptions = true;
		}
	}

	if ((argc < 2) || !ppArgv || !lpDiskPath || ShowSettingsDialog)
	{
		CSettingsDialog dlg;
		dlg.DoModal();
		return 0;
	}

	wchar_t wszKernelPath[512];
	if (!VirtualCDClient::Win32FileNameToKernelFileName(lpDiskPath, wszKernelPath, __countof(wszKernelPath)))
	{
		MessageBox(HWND_DESKTOP, _TR(IDS_BADIMGFN, "Invalid image file name!"), NULL, MB_ICONERROR);
		return 1;
	}

	VirtualCDClient clt;
	if (!clt.Valid())
	{
		MessageBox(HWND_DESKTOP, _TR(IDS_NOBUS, "Cannot connect to BazisVirtualCD.sys!"), NULL, MB_ICONERROR);
		return 1;
	}

	ActionStatus status;
	if (!clt.IsDiskConnected(wszKernelPath))
	{
		RegistryParams params;

		char driveLetter = 0;
		bool disableAutorun = false;
		if ((params.DriveLetterPolicy == drvlAskEveryTime) || ForceMountOptions)
		{
			CLetterSelectionDialog dlg(ForceMountOptions);
			if (dlg.DoModal() != IDOK)
				return 3;
			driveLetter = dlg.GetDriveLetter();
			disableAutorun = dlg.IsAutorunDisabled();
		}
		else
		{
			disableAutorun = params.DisableAutorun;
			if (params.DriveLetterPolicy == drvlStartingFromGiven)
			{
				DWORD dwMask = GetLogicalDrives();
				unsigned char ch = 'A' + params.StartingDriveLetterIndex;
				for (unsigned i = (1 << (ch - 'A')); ch <= 'Z'; ch++, i <<= 1)
					if (!(dwMask & i))
					{
						driveLetter = ch;
						break;
					}

			}
		}
		status = clt.ConnectDisk(wszKernelPath, driveLetter);//, disableAutorun);
	}
	else
		status = clt.DisconnectDisk(wszKernelPath);

	if (!status.Successful())
	{
		MessageBox(HWND_DESKTOP, status.GetMostInformativeText().c_str(), NULL, MB_ICONERROR);
		return 1;
	}
	return 0;
}